package common.tags;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import common.collect.Lists;
import common.log.Log;

abstract class TagList<K extends Tag> extends Tag {
	private List<K> list = Lists.newArrayList();

	void write(DataOutput output) throws IOException {
		output.writeInt(this.list.size());
		for(int z = 0; z < this.list.size(); z++) {
			this.list.get(z).write(output);
		}
	}

	void read(DataInput input, int depth, SizeTracker tracker) throws IOException {
		tracker.read(36);
		if(depth > 512)
			throw new RuntimeException("Objekt ist zu komplex, die Tiefe ist größer als 512");
		int len = input.readInt();
		tracker.read(4 * len);
		this.list = new ArrayList<K>(len);
		for(int z = 0; z < len; z++) {
			K tag = (K)Tag.createNewByType(this.getType());
			tag.read(input, depth + 1, tracker);
			this.list.add(tag);
		}
	}

	protected abstract byte getId();
	protected abstract byte getType();
	protected abstract TagList<K> createInstance();
	protected abstract K getDefault();

	public String toString() {
		StringBuilder sb = new StringBuilder("[");
		for(int z = 0; z < this.list.size(); z++) {
			if(z != 0)
				sb.append(',');
			sb.append(z).append(':').append(this.list.get(z));
		}
		return sb.append(']').toString();
	}

	protected void addElem(K tag) {
		this.list.add(tag);
	}

	protected void set(int index, K tag) {
		if(index < 0 || index >= this.list.size()) {
			Log.IO.error("Index außerhalb des Bereiches um Tag in Tag-Liste zu setzen");
			return;
		}
		this.list.set(index, tag);
	}

	public void remove(int index) {
		this.list.remove(index);
	}

	public boolean isEmpty() {
		return this.list.isEmpty();
	}

	protected K getElem(int index) {
		return index >= 0 && index < this.list.size() ? this.list.get(index) : this.getDefault();
	}

	public int size() {
		return this.list.size();
	}

	public Tag copy() {
		TagList<K> list = this.createInstance();
		for(K tag : this.list) {
			list.list.add((K)tag.copy());
		}
		return list;
	}

	public boolean equals(Object other) {
		return super.equals(other) && this.list.equals(((TagList)other).list);
	}

	public int hashCode() {
		return super.hashCode() ^ this.list.hashCode();
	}
}
