package common.village;

import java.util.Iterator;
import java.util.List;

import common.block.Block;
import common.block.Material;
import common.block.artificial.BlockDoor;
import common.collect.Lists;
import common.tags.TagObject;
import common.tags.TagObjectList;
import common.util.BlockPos;
import common.world.AWorldServer;

public class Village
{
    private final List<VillageDoorInfo> doors = Lists.<VillageDoorInfo>newArrayList();
    
    private BlockPos doorRange = BlockPos.ORIGIN;
    private BlockPos center = BlockPos.ORIGIN;
    private int radius;

//    public Village()
//    {
//    }
//
//    public Village(IWorldServer worldIn)
//    {
//        this.worldObj = worldIn;
//    }
//
//    public void setWorld(IWorldServer worldIn)
//    {
//        this.worldObj = worldIn;
//    }
    
    public void tick(AWorldServer world, int counter)
    {
//        this.tickCounter = counter;
        boolean mod = false;
        boolean reset = world.rand.zrange(50) == 0;
        Iterator<VillageDoorInfo> iterator = this.doors.iterator();

        while (iterator.hasNext())
        {
            VillageDoorInfo villagedoorinfo = (VillageDoorInfo)iterator.next();

            if (reset)
            {
                villagedoorinfo.resetDoorOpeningRestrictionCounter();
            }

            if (!this.isWoodDoor(world, villagedoorinfo.getDoorBlockPos()) || Math.abs(counter - villagedoorinfo.getInsidePosY()) > 1200)
            {
                this.doorRange = this.doorRange.subtract(villagedoorinfo.getDoorBlockPos());
                mod = true;
                villagedoorinfo.setIsDetachedFromVillageFlag(true);
                iterator.remove();
            }
        }

        if (mod)
        {
            this.updatePosition();
        }
    }

    public BlockPos getCenter()
    {
        return this.center;
    }

    public int getRadius()
    {
        return this.radius;
    }

//    public int getNumDoors()
//    {
//        return this.doors.size();
//    }

//    public int getSinceLastDoor()
//    {
//        return this.tickCounter - this.lastDoor;
//    }

//    public int getNumVillagers()
//    {
//        return this.numVillagers;
//    }

//    public boolean isInside(BlockPos pos)
//    {
//        return this.center.distanceSq(pos) < (double)(this.radius * this.radius);
//    }

    public List<VillageDoorInfo> getDoorList()
    {
        return this.doors;
    }

    public VillageDoorInfo getNearestDoor(BlockPos pos)
    {
        VillageDoorInfo villagedoorinfo = null;
        int i = Integer.MAX_VALUE;

        for (VillageDoorInfo villagedoorinfo1 : this.doors)
        {
            int j = villagedoorinfo1.getDistanceToDoorBlockSq(pos);

            if (j < i)
            {
                villagedoorinfo = villagedoorinfo1;
                i = j;
            }
        }

        return villagedoorinfo;
    }

    public VillageDoorInfo getDoorInfo(BlockPos pos)
    {
        VillageDoorInfo villagedoorinfo = null;
        int i = Integer.MAX_VALUE;

        for (VillageDoorInfo villagedoorinfo1 : this.doors)
        {
            int j = villagedoorinfo1.getDistanceToDoorBlockSq(pos);

            if (j > 256)
            {
                j = j * 1000;
            }
            else
            {
                j = villagedoorinfo1.getDoorOpeningRestrictionCounter();
            }

            if (j < i)
            {
                villagedoorinfo = villagedoorinfo1;
                i = j;
            }
        }

        return villagedoorinfo;
    }

    public VillageDoorInfo getExistedDoor(BlockPos doorBlock)
    {
        if (this.center.distanceSq(doorBlock) > (double)(this.radius * this.radius))
        {
            return null;
        }
        else
        {
            for (VillageDoorInfo villagedoorinfo : this.doors)
            {
                if (villagedoorinfo.getDoorBlockPos().getX() == doorBlock.getX() && villagedoorinfo.getDoorBlockPos().getZ() == doorBlock.getZ() && Math.abs(villagedoorinfo.getDoorBlockPos().getY() - doorBlock.getY()) <= 1)
                {
                    return villagedoorinfo;
                }
            }

            return null;
        }
    }

    public void addDoor(VillageDoorInfo doorInfo)
    {
        this.doors.add(doorInfo);
        this.doorRange = this.doorRange.add(doorInfo.getDoorBlockPos());
        this.updatePosition();
//        this.lastDoor = doorInfo.getInsidePosY();
    }

    public boolean isAnnihilated()
    {
        return this.doors.isEmpty();
    }

    private boolean isWoodDoor(AWorldServer world, BlockPos pos)
    {
        Block block = world.getState(pos).getBlock();
        return block instanceof BlockDoor ? block.getMaterial() == Material.WOOD : false;
    }

    private void updatePosition()
    {
        int i = this.doors.size();

        if (i == 0)
        {
            this.center = new BlockPos(0, 0, 0);
            this.radius = 0;
        }
        else
        {
            this.center = new BlockPos(this.doorRange.getX() / i, this.doorRange.getY() / i, this.doorRange.getZ() / i);
            int j = 0;

            for (VillageDoorInfo villagedoorinfo : this.doors)
            {
                j = Math.max(villagedoorinfo.getDistanceToDoorBlockSq(this.center), j);
            }

            this.radius = Math.max(32, (int)Math.sqrt((double)j) + 1);
        }
    }

    public void readVillageDataFromNBT(TagObject compound)
    {
//        this.numVillagers = compound.getInteger("PopSize");
        this.radius = compound.getInt("Radius");
//        this.lastDoor = compound.getInteger("Stable");
//        this.tickCounter = compound.getInteger("Tick");
//        this.noBreedTicks = compound.getInteger("MTick");
        this.center = new BlockPos(compound.getInt("CX"), compound.getInt("CY"), compound.getInt("CZ"));
        this.doorRange = new BlockPos(compound.getInt("ACX"), compound.getInt("ACY"), compound.getInt("ACZ"));
        TagObjectList nbttaglist = compound.getTagList("Doors");

        for (int i = 0; i < nbttaglist.size(); ++i)
        {
            TagObject nbttagcompound = nbttaglist.get(i);
            VillageDoorInfo villagedoorinfo = new VillageDoorInfo(new BlockPos(nbttagcompound.getInt("X"), nbttagcompound.getInt("Y"), nbttagcompound.getInt("Z")), nbttagcompound.getInt("IDX"), nbttagcompound.getInt("IDZ"), nbttagcompound.getInt("TS"));
            this.doors.add(villagedoorinfo);
        }
    }

    public void writeVillageDataToNBT(TagObject compound)
    {
//        compound.setInteger("PopSize", this.numVillagers);
        compound.setInt("Radius", this.radius);
//        compound.setInteger("Stable", this.lastDoor);
//        compound.setInteger("Tick", this.tickCounter);
//        compound.setInteger("MTick", this.noBreedTicks);
        compound.setInt("CX", this.center.getX());
        compound.setInt("CY", this.center.getY());
        compound.setInt("CZ", this.center.getZ());
        compound.setInt("ACX", this.doorRange.getX());
        compound.setInt("ACY", this.doorRange.getY());
        compound.setInt("ACZ", this.doorRange.getZ());
        TagObjectList nbttaglist = new TagObjectList();

        for (VillageDoorInfo villagedoorinfo : this.doors)
        {
            TagObject nbttagcompound = new TagObject();
            nbttagcompound.setInt("X", villagedoorinfo.getDoorBlockPos().getX());
            nbttagcompound.setInt("Y", villagedoorinfo.getDoorBlockPos().getY());
            nbttagcompound.setInt("Z", villagedoorinfo.getDoorBlockPos().getZ());
            nbttagcompound.setInt("IDX", villagedoorinfo.getInsideOffsetX());
            nbttagcompound.setInt("IDZ", villagedoorinfo.getInsideOffsetZ());
            nbttagcompound.setInt("TS", villagedoorinfo.getInsidePosY());
            nbttaglist.add(nbttagcompound);
        }

        compound.setTagList("Doors", nbttaglist);
    }
}
